import platform
import os


# ¡Instalación de apps necesarias!
run_in = platform.system()
platforms = {'Windows': 'Windows', 'Linux': 'Linux', 'Mac': 'Darwin'}  # Plataformas (Sí, Mac es Darwin XD)

#       Comandos:

# - Windows:
apps_commands_win = {'Jackett': 'winget install -e --id Jackett.Jackett', 'qBitTorrent': 'winget install -e --id qBittorrent.qBittorrent', 'Plex': 'winget install -e --id Plex.Plex', 'Python 3.9': 'winget install -e --id Python.Python.3.9'}




# INFO:
print('Tus aplicaciones se comenzarán a instalar. Una vez se comiencen a instalar le recomendamos no canelar la instalación, esto podría conllevar serios problemas.')

# Preguntamos al usuario si queire continuar, para que una vez comenzado,
# se trade de evitar la mayor cantidad de errores posibles.
continue_ask = input('¿Seguro quiere continuar? [Y] Yes, [N] No. | ---------->  ').upper()

# Pregunt si tienen Python, ya que es util  que tambíen se pueda instalar desde acá.
python_ask = input('¿Quiere instalar Python? [Y] Yes, [N] No. | ---------->  ').upper()

if continue_ask == 'Y':
    print('Los programas se comenzarán a instalar. No cancele nada por favor. Es probable que se pidan permisos de administrador.')

    if run_in == 'Windows':
        if python_ask == 'Y':
            for command_count, command in enumerate(list(apps_commands_win.values())[:-1]):
                print(command)
                i = os.system(command)

                if i == 0:
                    print(f' ======== Hemos instalado correctamente {list(apps_commands_win.keys())[command_count]} ======== ')

                else:
                    print(f'Hemos tenido un error al instalar {list(apps_commands_win.keys())[command_count]}')


        else:
            for command_count, command in enumerate(list(apps_commands_win.values())):
                print(command)
                i = os.system(command)

                if i == 0:
                    print(f' ======== Hemos instalado correctamente {list(apps_commands_win.keys())[command_count]} ======== ')

                else:
                    print(f'Hemos tenido un error al instalar {list(apps_commands_win.keys())[command_count]}')


    # TODO: Instalación para Linux y Mac.

else:
    print('Perfecto! Cuidate uwu.')
    exit()



from feedparser import exceptions
from qbittorrent import Client
import time
import os
import shutil


# Exeptions:
class CategoryContentError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return(repr(self.value))

class SeasonEpisodeError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return(repr(self.value))

class TorrentNotMatchError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return(repr(self.value))





def torrent_handler(torrent_name: str, original_name: str, route_moviesdb: str, torrent_type: str, qbtorrent_host: str, qbtorrent_user='admin', qbtorrent_pass='adminadmin', handler_time = 10, season = 'SXX', episode = 'EXX', content_release = '2005'):
    """
    ¡Usa ese módulo para estar pendiente de sí tus torrents ya descargaron! Recomendamos ejectuar esto en un nuevo hilo.
    
    
    Params
    =======
    torrent_name: str | Nombre del torrent a visualizar.

    original_name: str | El nombre oficial de la serie/pelicula. Esto lo usamos para renombrar archivos, crear carpetas etc.

    route_moviesdb: str | Ruta definitiva para el contenido. Esta ruta es la que tomará plex para ver el contenido. Lo ideal sería que una vez  establecida no fuera cambiada, por tanto, ten en cuenta esto.

    torrent_type: str | El tipo de contenido que está intentando rastrear. Ese parametro soporta SOLO dos valores: 'tv' o 'movie'

    qbtorrent_host: str | El host donde esta corriento tu qBittorrent WEB.

    qbtorrent_user: str | Usuario admin en tu qBitTorrrent!

    qbtorrent_pass: str | Contraseña del usuario admin de tu QBitTorrent.

    handler_time: float | Tiempo de espera sobre el cual se harán las peticiones al estado de la descarga, en segundos. Recomiendo no dejar un numero tan alto (como 60), ni tan bajo (como 3).

    season: str | Si el parametro de "torrent_type" es "tv", es necesario especificar la temporada con el formato SXX, donde "X" es el número de temporada. Ej: S01.

    episode: str | Si el parametro de "torrent_type" es "tv", es necesario especificar el episodio con el formato EXX, donde "X" es el número de episodio. Ej: E02.

    content_release: str | Fecha de estreno del contenido dado!

    """
    
    torrent_category = ['tv', 'movie']

    if torrent_type not in torrent_category:
        raise CategoryContentError('(error 01) Tu "torrent_type" no es valido. Recuerda que, este parametro solo soporta dos valores: "tv" o "movie".')
        exit()
        
    if torrent_type == 'tv' and season == 'SXX' or torrent_type == 'tv' and episode == 'EXX':
        raise SeasonEpisodeError('(error 02) Por favor configure los parametros de "season", "episode".')



    # Torrent Name!
    name = torrent_name

    # Login qBitTorrent
    qb = Client(qbtorrent_host)  # Nos conectamos al qBittorrent.
    qb.login(qbtorrent_user, qbtorrent_pass)  # Login.

    # Con el handler en sí, cada 10 s haremos una petición a qBitTorrent server para ver y manejar la descarga. De aquí solo sacaremos el hash de la descarga (por si la necesitamos), el estado actual y el path, que es donde se está descargando.

    # Esta sub función la usamos para obtener el estado del torrent. Esto lo sacamos en una función aparte, ya que, es lo que por separado, toca ejectutar constantemente para obtener SIEMRPE el estado del torrent.
    def get_state():
        my_hash = ''
        state = ''
        path = ''
        name_torrent = ''
        content_path = ''


        torrents_list = qb.torrents()
        
        torrents_list_names = []

        for name_torrent_s in torrents_list:
            torrents_list_names.append(name_torrent_s['name'])

        
        if torrent_name not in torrents_list_names:
                raise TorrentNotMatchError('(error 03) El nombre de tu torrent no coincide ninguno en descarga.')
                exit()

        # Vemos si el nombre proporcionado coincide con alguno de la lista actual del qBitTorrent.
        for download in torrents_list:

            # ¡Viendo un poco los torrents de diferentes descarsgas, algunas traen dobles espacios, ni puta idea de por qué, pero para solucionar eso básicmante eliminamos uno de esos espacios para que puedan coincidir perfectamente!
            if download['name'].replace('  ', ' ') != name:
                my_hash = download['hash']  # Extraemos el hash.
                state = download['state']  # Extraemos el estado actual.
                path = download['save_path']  # Extraemos la ruta de descarga.
                name_torrent = download['name']  # Extraemos el nombre del torrent.
                content_path = download['content_path']

                continue



            elif download['name'] != name:
                my_hash = download['hash']
                state = download['state']
                path = download['save_path']
                name_torrent = download['name']
                content_path = download['content_path']

            else:
                my_hash = download['hash']
                state = download['state']
                path = download['save_path']
                name_torrent = download['name']
                content_path = download['content_path']


            break

        return [state, path, my_hash, name_torrent, content_path]


    # Handler en sí.
    while True:
        handler_state = get_state()

        # torrent_files = qb.get_torrent_files(handler_state[2])
        # print(f' - {torrent_files} -')


        print(f'====================================\n Siguiendo: {handler_state[2]}')
        print(f' {handler_state[0]} \n====================================\n\n\n')
        
        base_route = handler_state[1]

        if base_route[-1] == '/' or base_route[-1] == '\\':
            base_route = base_route[:-1]
            
            
        #print(handler_state[4])
        #print(base_route)
        

        # Acá es donde empezamos a manejar los archivos. ¡Con esto, ya verificamos que descargó y simplemente tenemos que idear una buena forma de detectar si es una serie o es una pelicula! (probablemente OMBI ayude)
        if handler_state[0] == 'stalledUP' or handler_state[0] is None or handler_state[0] == 'uploading':  # Esto quiere decir que está seedeando, terminó de descargar o está "subiendo".
            
            delete_torrent = qb.delete(handler_state[2])


            # download_route = handler_state[1]  # ¡Ruta donde se descargó el torrent!
            
            print(f'---------> > > {handler_state[4]} < < <----------')
            
            if os.path.isdir(handler_state[4]) == True:
                folder_download = os.listdir(f'{base_route}/{handler_state[3]}')  # Lista de archivos de donde se descargó el torrent.
            else:
                folder_download = [os.path.basename(handler_state[4])]
                
            
            


            delete_files = []

            for filec, fileff in enumerate(folder_download):  # Saco todos los archivos".html", solo saco esto porque estoy seguro de que aquí no hay nada más. No va a haber ningún otro archivo. Estos dos, serán acompañados por la carpeta donde estarán los archivos de video, en el caso de que se haya encontrado algo para descargar.
                if '.html' in fileff or '.txt' in fileff or '.url' in fileff:
                    delete_files.append(fileff)  # Sacamos eso de la lista. ¿Por qué? Luego lo usamos para mover los archivos, y no queremos que se muevan archivos basura.
                    # folder_download.pop(filec)

            # Con esto, eliminamos los archivos basura que no necesitamos.
            for file_to_delete in delete_files:
                os.remove(f'{base_route}/{handler_state[3]}/{file_to_delete}')


            # Creo una lista con SOLO los archivos "mkv"
            video_files = []
            video_formats = []
            
            for video_file in folder_download:
                if '.mkv' in video_file or '.MKV' in video_file or '.avi' in video_file or '.AVI' in video_file or '.mp4' in video_file or '.MP4' in video_file:
                    vf = video_file[::-1]
                    vf = video_file[:vf.find('.')][::-1]
                    video_formats.append(vf)
                    
                    video_files.append(video_file)

            
            print(f' ======> {video_formats}')
            # Renombrando archivos!
            if len(video_files) == 1:

                if torrent_type == 'tv':

                    if os.path.isdir(handler_state[4]):
                        tv_name = f'{base_route[1]}/{handler_state[3]}'

                    else:
                        tv_name = f'{handler_state[1]}'



                    season_remake = season[1:]
                    episode_remake = episode[1:]

                    if season_remake[0] == '0' and len(season_remake) == 2:
                        season_remake = season_remake[1:]
                        
                    if route_moviesdb[-1] == '/' or route_moviesdb[-1] == '\\':
                        route_moviesdb = route_moviesdb[:-1]
                        
                    new_tv_name_route = f'{route_moviesdb}/tv/{original_name}'
                    new_tv_name = f'{route_moviesdb}/tv/{original_name}/{season_remake}x{episode_remake}.{video_formats[0]}'
                    
                    print('Creando carpetas tv y movies.')
                    try:
                        os.mkdir(f'{route_moviesdb}/tv')
                        os.mkdir(f'{route_moviesdb}/movies')
            
                    except:
                        pass
                    
                    
                    print('Creando carpeta de la serie.')
                    try:
                        os.mkdir(new_tv_name_route)

                    except:
                        pass

                    
                    


                    print('renombrando...')
                    os.rename(f'{tv_name}/{video_files[0]}', f'{tv_name}/{season_remake}x{episode_remake}.{video_formats[0]}')
                    time.sleep(2)
                    
                    
                    print('moviendo')
                    shutil.move(f'{tv_name}/{season_remake}x{episode_remake}.{video_formats}', new_tv_name)



                else:
                    print('movie')
                    if os.path.isdir(handler_state[4]):
                        movie_name = f'{base_route}/{handler_state[3]}'

                    else:
                        movie_name = f'{base_route}'



                    new_movie_name = f'{route_moviesdb}/movies/{original_name} ({content_release}).{video_formats}'

                    print('Creando rutas para pelis.')
                    try:
                        os.mkdir(f'{route_moviesdb}/tv')
                        os.mkdir(f'{route_moviesdb}/movies')
            
                    except:
                        pass

                    print(f'{movie_name}/{video_files[0]}')
                    os.rename(f'{movie_name}/{video_files[0]}', f'{movie_name}/{original_name} ({content_release}).{video_formats}')

                    shutil.move(f'{movie_name}/{original_name} ({content_release}).{video_formats}', new_movie_name)

            



            # Creando carpetas necesarias!
            if route_moviesdb[-1] == '/' or route_moviesdb[-1] == '/':
                route_moviesdb = route_moviesdb[:-1]

                #f'{route_moviesdb}/{original_name} {season}{episode}'

            print(f'Se ha hecho la descarga de {original_name}')
            break

            

        # ¡Para estar pendiente de los torrents, pero que no esté ejecutandoce siempre, lo ponemos a esperar un tiempo, para que luego vuelva a ver el estado!
        time.sleep(handler_time)

        # Riverdale - Temporada 1 [HDTV 720p][Cap.105][AC3 5.1 Español Castellano]

        # torrent_name: str, original_name: str, route_moviesdb: str, torrent_type: str, qbtorrent_host: str, qbtorrent_user='admin', qbtorrent_pass='adminadmin', handler_time = 10, season = 'SXX', episode = 'EXX', content_release = '2005'




import requests
from datetime import date

# ombi_login = requests.post('http://mikoin.sytes.net:5000/api/v1/Logging')


def ombi_requests(ombi_host: str, ombi_apikey: str):
    """
    Usa este módulo como handler de tu servidor ombi y resivir las peliculas o series que se piden. Por ahora solo soporta las series, pero se está trabajando para hacerlo compatible con peliculas de igual forma.


    Params
    =======
    ombi_host: str | Dirección url donde está corriendo tu OMBI.

    ombi_apikey: str | Clave de la api de OMBI. Importante para efectuar correctamente las consultas.

    """
    

    ombi_header = {'ApiKey': ombi_apikey}  # Header que se envia al ombi y que es NECESARIO para que pueda ejecutar las consultas.

    # Aquí hacemos la petición a la API.

    if ombi_host[-1] == '/':
        ombi_tv = requests.get(f'{ombi_host}api/v1/Request/tv', params=ombi_header)  # Consulta que devuelve las series.
        ombi_movies = requests.get(f'{ombi_host}api/v1/Request/movie', params=ombi_header)  # Consulta que devuelve las peliculas sin aceptar.

    else:
        ombi_tv = requests.get(f'{ombi_host}/api/v1/Request/tv', params=ombi_header)  # Consulta que devuelve las series.
        ombi_movies = requests.get(f'{ombi_host}/api/v1/Request/movie', params=ombi_header)  # Consulta que devuelve las peliculas sin aceptar.

    
    
    if ombi_tv.status_code == 404:
        raise ValueError('(OmbiHostError, error 01) Es posible que la URL de donde está corriendo tu servidor de ombi sea incorrecta, asegurese de que esté en el puerto correcto!')

    elif ombi_tv.status_code == 401:
        raise ValueError('(OmbiApiKeyError, error 02) La API Key proporcionada es erronea. Asegurese de proporcionar la correcta. Si no está seguro de como obtenerla, vaya a su Ombi > Settings > Configuration > General > Api Key')


    #try:
    tv_requests = ombi_tv.json()  # Esto nos devolvería ya las series que se han pedido.
    #except json.decoder.JSONDecodeError:
    #    print('cero')

    # Claramente en formato JSON.
    movie_requests = ombi_movies.json()
    # print(movie_requests)

    # PARA LAS PETICIONES DE SERIES.
    # No son lo mismo que las peliculas, ya que en ocaciones, algunos
    # capitulos de la serie no se han publicado.

    # Lista de contenido en cola.
    cola = []

    capitulos_publicados = []
    show_incomplete = []

    complete: bool = True
    

    # Añadimos la serie a la cola!
    for show in tv_requests:
        show_status = show['childRequests'][0]['requestStatus'].split('.')[1]
        show_title = show['title'] # TODO: Converti nombre.
        show_id = show['id']
        


        show_seasons = show['totalSeasons']
        show_S_requests = show['childRequests'][0]['seasonRequests']
        show_request_ombi = show['childRequests'][0]['parentRequestId']

        show_tvId = show['externalProviderId']
        # print(show)
        
        get_serie_info = requests.get(f'https://api.themoviedb.org/3/tv/{show_tvId}?api_key=5eb7e21201ae0b13d5e4f992ee9d5471&language=es-ES')  # Se hace la petición con mi propia API Key, espero no conlleve un problema XDDDD.

        serie_info = get_serie_info.json()  # Esto nos devuelve un diccionario.
        show_title = serie_info['name']  # El titulo de la serie ya en español (en el caso de que esté).
        
        show_release = serie_info['first_air_date'].split('-')[0]

        complete = True

        for season in show['childRequests'][0]['seasonRequests']:
            charapters = []

            season_number = season['seasonNumber']

            if season_number >= 10:
                season_number = f"S{season['seasonNumber']}"

            else:
                season_number = f"S0{season['seasonNumber']}"

            for episode in season['episodes']:
                episode_number = episode['episodeNumber']

                if episode_number >= 10:
                    episode_number = f"E{episode['episodeNumber']}"

                else:
                    episode_number = f"E0{episode['episodeNumber']}"

                today = str(date.today())
                today_split = today.split('-')  # Spliteado en: Año/Mes/Día.

                pub = episode['airDate'].split('T')[0]
                pub_split = pub.split('-')  # Spliteado en: Año/Mes/Día.

                episode_se = f'{season_number}{episode_number}'

                if pub_split[0] <= today_split[0]:  # Si el año es menor o el mismo que el actual:

                    if pub_split[0] == today_split[0]:  # Si el año es el mismo,
                        # el mes de pub tiene que ser IGUAL o menor al actual.
                        if pub_split[1] <= today_split[1]:

                            if pub_split[1] == today_split[1]:
                                if pub_split[2] < today_split[2]:  # Si el día de publicación es menor que el actual:
                                    capitulos_publicados.append(True)
                                    complete = True

                                else:
                                    capitulos_publicados.append(False)
                                    complete = False


                            else:
                                capitulos_publicados.append(True)
                                complete = True


                        else:
                            capitulos_publicados.append(False)
                            complete = False
                    else:
                        capitulos_publicados.append(True)
                        complete = True

                    charapters.append(f'{show_title} {episode_se}')


            if complete:
                show_incomplete.insert(0, f'{show_title} {season_number}')

            else:
                for i in charapters:
                    show_incomplete.append(i)

        charapters.append({'contentType': 'tv', 'showId': show_id, 'dbId': show_tvId, 'title': show_title, 'release': show_release})
        
        # Acá vamos a hacer una excepción con Dahmer – Monster: The Jeffrey Dahmer Story, ya que es conocida simplemente como Dahmer. Esto supongo que lo iré haciendo con diferentes series según convenga.
        charapters_except = []
        for c in charapters[:-1]:
            if isinstance(c, str) and 'Monstruo: La historia de Jeffrey Dahmer' in c:
                c = c.replace('Monstruo: La historia de Jeffrey Dahmer', 'Dahmer')
                charapters_except.append(c)

        


        if len(charapters_except) == 0:
            cola.append(charapters)
            
        else:
            charapters_except.append(charapters[-1])
            cola.append(charapters_except)


    movies = []


    for movie in movie_requests:
        consult_movie = requests.get(f'https://api.themoviedb.org/3/movie/{movie["theMovieDbId"]}?api_key=5eb7e21201ae0b13d5e4f992ee9d5471&language=es-ES').json()
        
        movie_title_es = consult_movie['title']
        movie_id = movie['id']
        movie_release = movie['releaseDate'].split('-')[0]
        

        # movie_append = [movie_title_es, movie['theMovieDbId'], ['movie', movie_id]]
        movies.append([movie_title_es, {'contentType': 'movie', 'showId': movie_id, 'dbId': movie['theMovieDbId'], 'title': movie_title_es, 'release': movie_release}])

        # Sacamos el titulo y el ID de TMDB. Esto mismo debo hacer con las series, no es complicado.
        # movies.append(movie_append)



    for movie in movies:
        cola.append(movie)

 
    return cola  # Esto devuelve una lista de listas con la serie! Un ejemplo sería el siguiente: [['La casa del Dragón S01E05', 234252, ['tv', 109]]]!! Para cada lista, los dos últimos indices pertenecen al ID del TheMovieDb y el ID del request del Ombi. Esto es útil para poder eliminar la requests desde API de OMBI.

    # Eso se tiene que hacer en bucle desde fuera.
    # Si se ejecuta este módulo una sola vez, solo obtiene la información de ese momento,
    # y, por tanto, no es conveniente dependiendo qué casos.

    # TODO: Crear función para extraer las peliculas también.

    


# Función para eliminar el requests de Ombi.
def ombi_delete(ombi_request_id: str, ombi_host: str, ombi_apikey: str, content_type:str):
    """
    Elimina una petición dado su requestId. Si se utiliza la función anterior, esta se devuelve
    en el último índice de cada lista. Esto lo usamos para que, después de aceptar y
    poner a descargar la serie/pelicula, no se quede en el apartado de "espera".
    
    
    Params
    =======
    Usa este módulo como handler de tu servidor ombi y resivir las peliculas o series que se piden. Por ahora solo soporta las series, pero se está trabajando para hacerlo compatible con peliculas de igual forma.


    Params
    =======
    ombi_request_id: (str) | Clave de la api de OMBI. Importante para efectuar correctamente las consultas.

    ombi_host: str | Dirección url donde está corriendo tu OMBI.

    ombi_apikey: str | Clave de la api de OMBI. Importante para efectuar correctamente las consultas.

    content_type: str | El tipo de contenido del que se va a borrar. Recibe solo dos posibles parametros: "movie" o "tv".

    """
    
    # ===========================================
    headers = {'content-type': 'application/json'}
    ombi_header = {'ApiKey': ombi_apikey}
    
    


    if ombi_host[-1] == '/':
        ombi_host = ombi_host[:-1]
    else:
        pass
    

    # Aquí hacemos la petición a la API.
    if content_type == 'tv':
        consult = requests.delete(f'{ombi_host}/api/v1/Request/tv/{ombi_request_id}',params=ombi_header)

        if consult.status_code == 200:
            return 'Eliminación completa'

        else:
            return 'Algo ha sucedido, puede que no este pasando el request id correcto.'

    else:
        consult = requests.delete(f'{ombi_host}/api/v1/Request/movie/{ombi_request_id}',  params = ombi_header, headers = headers, data = ombi_header)  # TODO: ARREGLAR DE QUE SE ELIMINEN LA PELICULAS SOLICITADAS. Funciona con las series pero no con las peliculas por alguna razón.No tiene sentido. Puede ser por el desactualizado server, pero debemos esperar.


        print(consult.headers)
       
        if consult.status_code == 200:
            return 'Eliminación completa'

        else:
            return 'Algo ha sucedido, puede que no este pasando el request id correcto.'


# Libraries
import feedparser
import requests
import os
import time

from qbittorrent import Client



def download(search: str, jacket_host: str, jacket_apiKey: str, qbtorrent_host: str, qbtorrent_user: str, qbtorrent_pass: str, download_path: str, max_size = 2000, low_discard = True):
    """
    Descarga los torrents de las peliculas con ayuda de este módulo.

    Params
    ====
    search: str | Nombre de la serie o película a buscar!

    jackett_host: str | El host dond está corriendo tu servidor de Jackett.

    jackett_apiKey: str | La API KEY de tu Jackett! La puedes encontrar arriba derecha de tú Jackett.

    qbtorrent_host: str | El host donde esta corriento tu qBitTorrent WEB.

    qbtorrent_user: str | Usuario admin en tu qBitTorrrent!

    qbtorrent_pass: str | Contraseña de tu usuario en tu qBitTorrent!

    max_size: int | Peso máximo (en MB) que podrán tener los archivos.

    low_discard: bool | Si desea que se descarte el contenido en 720p, active esto!

    download_path: str | Ruta donde se descargarán los archivos virgenes, sin haberlos procesado y renombrado, por tanto, no des la ruta defenitiva. RUTA ABSOLUTA!!!!
    """



    # Configuración inicial del string debúsqueda.
    busqueda = search.replace(' ', '+')
    
    # Configuraciones básicas. Es importante que, para empezar, se tiene que tener los trackers ya puestos en jackett.


    # A continuación extraigo la información de la serie (temporada y episodio) guiandome del formato SXXEXX que ya conociamos.
    serie_number_info = search.split(' ')[-1].upper()  # -1 porque,la temporada y episodio de la serie queda a lo último.


    parameters: str

    if 'S' in serie_number_info[0] and 'E' in serie_number_info[1:]:  # Desde acá, descartamos directamente el indice 0, ya que ya se validó y no lo necesitamos.

        # Extraemos el numero de temporada y episodio.
        season_number = serie_number_info[1:serie_number_info.find('E')]
        episode_number = serie_number_info[serie_number_info.find('E') + 1:]


        parameters = f'api/v2.0/indexers/all/results/torznab/api?apikey={jacket_apiKey}&cat=5000,5030,5040,5070,2000,2030,2040,2045,2070&t=search&q={busqueda[:busqueda.find(serie_number_info)]}&season={season_number}&ep={episode_number}'

        

    else:
        parameters = f'api/v2.0/indexers/all/results/torznab/api?apikey={jacket_apiKey}&t=search&q={busqueda}'
        
        

    # Validación de sí la url pasada contiene la típica "/" al fínal. Sí es así, se la eliminamos.
    if jacket_host[-1] == '/':
        url_consult = f'{jacket_host[:-1]}/{parameters}'  # ¡Url lista para consulta!

    else:
        url_consult = f'{jacket_host}/{parameters}'  # ¡Url lista para consulta!


    print(f'Haciendo la busqueda de {search}.')



    # Accediendo!
    feed = feedparser.parse(url_consult)  # Se lee el RSS. ¡Devuelve una lista de diccionarios con los resultados!


    # Rastreo el estado 404 por si se ingresa mal el host del jackett!
    if feed['status'] == 404:
        raise Exception('(JackettHostError, error 02) Hay un erorr con la url del Jackett. Revisala. Puede que se deba a que esté mal redactada, que no esté corriendo el servidor o que esté en otro puerto.')



    torrents = []  # Diccionario para guardar los torrents


    for torrent in feed['entries']:
        torrents.append([torrent['size'], torrent['link'], torrent['title']])

        

    max_size_bytes = max_size * 1000000


    torrents_definitive_list = []
    

    # TODO: si se prefiere que los torrents que contengan 720, comentar las siguientes lineas de código.
    for torrents_info_count, torrents_info in enumerate(torrents):
        # Sí el nombre del torrent no contiene "720" y, sí el peso del torrent no supera el ya pasado a la función como parametro:
        if low_discard == True:
            if '720' not in torrents_info[-1] and int(torrents_info[0]) <= max_size_bytes:
                torrents_definitive_list.append(torrents[torrents_info_count])

        else:
            if int(torrents_info[0]) <= max_size_bytes:
                torrents_definitive_list.append(torrents[torrents_info_count])


    if len(torrents_definitive_list) == 0:
        print('(NoContentError, error 01) No se ha podido obtener  ninguna pelicula/serie para tu búsqueda, puede que se deba a error en la busqueda, que sea demasiado nuevo el contenido o que la filtración por los parametros causó que no quedara nada.')


    # Ahora sí, descarga de los torrents en sí.

    try:
        f_torrent = torrents_definitive_list[0] # Tomamos el primero de la lista. Esto aún se puede barajar, y ver si lo totamos otro torrent por diferente razón. Recordar que por lo menos, el RSS del jackett no devuelve los seeders ni peers para obtener más metadata, por tanto, nos vemos por este lado restringidos.

        # Config torrents var's.
        torrent_link = f_torrent[1]
        torrent_size = f_torrent[0]
        torrent_name = f_torrent[2]
        


        try:
            # Conexión al servidor de qBitTorrent!
            global qb
            qb = Client(qbtorrent_host)



        # Excepción si la url pasa está mal.
        except requests.exceptions.ConnectionError:
            print('(qBitTorrentHostError, error 03) El link del host de qBitTorrent es erroneo. Puede que tu servidor esté corriendo sobre otro puerto, esté apagado o hayas escrito mal la URL. Revisalo!')
            exit()
        
        # Iniciamos sesión en qBitTorrent para empezar a usarlo.
        qbtorrent_login = qb.login(qbtorrent_user, qbtorrent_pass)
        # print(qbtorrent_login)
        


        # Manejo de contraseña/usuario incorrecto.
        if qbtorrent_login == 'Fails.':
            raise ValueError('(qBTorrentCredentialsError, error 04) Al parecer, las credenciales de tu qBitTorrent (usuario o contraseña) están mal. Como recordatorio: la contraseña por defecto del qBitTorrent es "adminadmin", y por otro lado, el usuario es "admin".')
        



        # TODO: Extraer información del torrent antes de descargarlo.

        # Descarga final del torrent!
        t_d = qb.download_from_link(torrent_link, savepath = download_path)  # Si la ruta está mal, directamente se descarga en descargas.
        print(f'\nLa descarga de {torrent_name} comenzó.')
        time.sleep(3)

        active_torrents = qb.torrents(sort = 'added_on')  # Devielve TODOS los torrents activos ordenados por orden de añadido!
        re_active = []  # Lista para rehacer la lista de los torrents activos.


        for at in active_torrents:
            re_active.append({'added_on': at['added_on'], 'hash': at['hash'], 'name': at['name']})
        

        active_torrents_sort = re_active[::-1]
        
        actual_torrent = active_torrents_sort[0]
        torrent_name = actual_torrent['name']

        

        return torrent_name




    except IndexError:
        pass




from setuptools import setup


from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text('utf-8')




setup(
    name = 'MovieTool',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    packages = ['MovieTool'],
    version = '3.5',
    license = 'MIT',
    description = "Descarga tus peliculas y series, automatiza todo y crea tu propio Netflix (hecho con fines de aprendizaje) 100% funcional (y con más contenido que el de Netflix) haciendo uso de Plex, Ombi y otras tecnologías más. ⭐",
    author = 'ElHaban3ro',
    author_email = 'habanferd@gmail.com',
    url = 'https://github.com/ElHaban3ro/MovieTool',
    keywords = ['python', 'torrent', 'torrent-management', 'movies', 'plex', 'hbo', 'series', 'netflix', 'ombi', 'peliculas'],
    classifiers = [
        'Programming Language :: Python :: 3.10'
    ],
    install_requires=["pandas==1.4.4", "python-qbittorrent==0.4.2", "lxml==4.9.1", "feedparser==6.0.10"]
)

